// WIP

/**
 * BrickLink Wanted List Exporter PLUS
 * Extract wanted list data from a wanted list edit page, generate the proper 
 * XML for Bricklink import and copy it in the clipboard. 
 * The list is extracted after that the page has been reloaded adding 
 * &wantedSize=1000 to include all the parts of a list in a single page. 
 * This version accept more complex URLs for the wanted list instead than
 * basic URLs only. 
 * NOTE; with the recent changes in Bricklink, you have to switch to the "Classic"
 *       wanted list view to use this script. 
 * 
 * @name           BrickLink Wanted List Exporter
 * @description    Export BrickLink Wanted Lis
 * @author         Original script by Ng Hun Yang
 * @version        1.1
 *
 */ 

/* Tested on Firefox 47.0 */


// Constants //////////////////////////////////////////////////////////////////////////////////////
const OUTPUT_TYPE = 0;  // 0: Clipboard - 1: new Tab - 2: new Window
const PAGE_URL = window.content.location.href;
const PAGE_DOC = FireGestures.sourceNode.ownerDocument; 
const BASE_URL = "http://www.bricklink.com";
const WANTED_RESP = /wantedMoreID=([0-9]+)/;
const PAGE_RESP = /^(http|https)\:[/]{2}www\.bricklink\.com[/]{1}wantedDetail\.asp[?]showOld=Y[&]catType=(S|P|M|B|G|C|I|O|U)[&]hideItems=[Y|N][&]wantedMoreID=[0-9]+$/ 
const PARTS_RESP = /\?(\w+)=([0-9a-zA-Z-]+)[^=]+=([0-9]+)$/
const OTHER_RESP = /\?(\w+)=([0-9a-zA-Z-]+)$/
const CLEAR_URL_RESP = /(a=[a-z][&]|[&]wantedSize=.*)/g;
const CLIP_MESSAGE = true;
const CLIP_CONFIRM_TEXT = "Data copied in the Clipboard"; 

// Variables //////////////////////////////////////////////////////////////////////////////////////
var doc = PAGE_DOC;

// Functions //////////////////////////////////////////////////////////////////////////////////////

// Escape a string from tags
function escTags(str) {
  return str.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
}

// Get the wanted lists
function getWantedListData(extDoc) {
  var urlResp = /[/]wantedView\.asp[?]wantedMoreID=/;
  
  // Get Wanted List ID
  var matches = PAGE_URL.match(WANTED_RESP);
  var wantedID = matches[1];

  // Get Wanted List Name
  var wantedName = "";
  var anchors = extDoc.getElementsByTagName("a");
  for (var i=0 ; i < anchors.length ; i++) {
    if (anchors[i] &&
      anchors[i].hasAttribute("href") ) {
      url = anchors[i].getAttribute("href");
      // Look for the desired link
      if (urlResp.test(url)) {
        wantedName = anchors[i].textContent;
        break;
      }
    }
  }
  
  var wantedData = []; 
  var wantedData = { "id": wantedID, "name": wantedName};

  return wantedData;
}

// Get parts from the page
function getParts(extDoc) {
  function get_elm_id(row) {
    if(row.children.length < 2)
      return false;

    var hElm = row.children[1].querySelector("a");
    if(!hElm)
      return false;

    if(!hElm.getAttribute("href").match(/\?wantedID=(\d+)$/))
      return false;

    return RegExp.$1;
  }

  function get_part_info(row) {
    if(row.children.length < 3)
      return false;

    var hElm = row.children[2].querySelector("a");
    if(!hElm) 
      return false;

      var tType = "";
      var tId = "";
      var tColour = "";

    if(hElm.getAttribute("href").match(PARTS_RESP)) {
      tType = RegExp.$1;
      tId = RegExp.$2;
      tColour = RegExp.$3;
    } else if(hElm.getAttribute("href").match(OTHER_RESP)) {
      tType = RegExp.$1;
      tId = RegExp.$2;
      tColour = "0";
    } else {
      return false;
    }

    return { type: tType, id: tId, colour: tColour};  
  }

  
  // Entry point 
  var form = extDoc.querySelector("form[name=wantedForm]");
  var tbl = form.parentNode;
  var rows = tbl.querySelectorAll("tr");
  
  var list = [];

  for(var i = 1; i < rows.length; ++i) {
    var row = rows[i];

    var part_info = get_part_info(row);
    if(!part_info)
      continue;

    row = rows[i + 1];

    var notify = row.children[0].querySelectorAll("input").item(1).checked ? "Y" : "N";

    var hElm = row.children[1].querySelector("select");
    var cond = hElm.value;

    var min_qty = row.children[2].querySelectorAll("input").item(1).value;
    var max_price = row.children[3].querySelectorAll("input").item(1).value;
    var remarks = row.children[4].querySelector("input").value;

    list.push({
      part_type: part_info.type,
      part_id: part_info.id,
      color: part_info.colour, 
      cond: cond,
      min_qty: min_qty,
      max_price: max_price,
      remarks: remarks,
      notify: notify
      });
    }

  return list;
}

// Get a single wanted list
function genWantedList(list, wanted_list) {
  function emit(tag, v) {
    return "<" + tag + ">" + v + "</" + tag + ">";
    }

  // Entry point
  var out_list = [];

  for(var i = 0; i < list.length; ++i) {
    var elm = list[i];
    var str = "";
    str += emit("ITEMTYPE", elm.part_type);
    str += emit("ITEMID", elm.part_id);
    str += emit("COLOR", elm.color);
    str += emit("CONDITION", elm.cond);
    str += emit("MINQTY", escTags(elm.min_qty));

    if(elm.max_price != "")
      str += emit("MAXPRICE", escTags(elm.max_price));

    if(elm.remarks != "")
      str += emit("REMARKS", escTags(elm.remarks));

    if(elm.notify != "N")
      str += emit("NOTIFY", elm.notify);

    if(wanted_list != 0)
      str += emit("WANTEDLISTID", wanted_list);

    out_list.push(emit("ITEM", str));
    }

  return emit("INVENTORY", "\n" + out_list.join("\n") + "\n");
}

// Open the list in a new tab
function newTabList(xmlData, wanted_list) {
  var title = wanted_list.name + " (" + wanted_list.id + ")";
  var new_tab = gBrowser.getBrowserForTab(gBrowser.addTab(""));

  var head = "<html><head><title>" + title + "</title></head><body><pre>" 
  var tail = "</pre></body></html>"
  
  new_tab.addEventListener("load", function () {
    new_tab.contentDocument.body.innerHTML = head + escTags(xmlData) + tail;
  }, true);
  
  gBrowser.tabContainer.advanceSelectedTab(1, true);
}

// Opens the list on a new window
function newWinList(xmlData, wanted_list) {
  var title = wanted_list.name + " (" + wanted_list.id + ")";
  var new_win = window.open("", title); 

  var head = "<html><head><title>" + title + "</title></head><body><pre>" 
  var tail = "</pre></body></html>"

  var new_doc = new_win.document;
  new_doc.body.innerHTML = head + escTags(xmlData) + tail;
  new_doc.close();
}

// Copy the list on the clipboard
function clipboardList(xmlData) {
  var clipboard = Cc["@mozilla.org/widget/clipboardhelper;1"]
                .getService(Ci.nsIClipboardHelper);
  clipboard.copyString(xmlData);

  if (CLIP_MESSAGE) {
    var extDiv = doc.createElement("div");
    extDiv.id = "lightbox";
    extDiv.style = "position: fixed; z-index: 999; width: 100%; height: 100%; left: 0; top: 0; text-align: center; background-color: rgba(0,79,45,0.4);"; 
    extDiv.addEventListener("click", function(){doc.getElementById("lightbox").parentNode.removeChild(extDiv); });

    var div = doc.createElement("div"); 
    div.id = "confirmBox";
    div.style = "position: absolute; left: 50%; top: 50%; transform: translate(-50%,-50%); border: 2px solid gray; padding: 20px; background-color: Lavender; font-size: 1.5em; font-weight: bold; ";
    var text = doc.createTextNode(CLIP_CONFIRM_TEXT); 

    div.appendChild(text); 
    extDiv.appendChild(div); 
    doc.body.appendChild(extDiv);
  }
}

// Emit the wanted list
function emitWantedList(extDoc) {

  var wanted_list = getWantedListData(extDoc); 
  var xmlData = genWantedList(getParts(extDoc), wanted_list.id); 

  switch (OUTPUT_TYPE) {
    case 1:
        newTabList(xmlData, wanted_list);
        break;
    case 2:
        newWinList(xmlData, wanted_list);
        break;
    default:
        clipboardList(xmlData);
  } 
}

// Load a webpage (specified in the parameter theURL) in a varible
// as a document. To do that an asyncronous XMLHttpRequest is used. 
function loadHTMLPage(theURL) {
  xhr = new XMLHttpRequest();
  xhr.open("GET", theURL , true);
  xhr.responseType = "document";
  xhr.onload = function (e) {
    if (xhr.readyState === 4) {        // Callback
      if (xhr.status === 200) {
        var extDoc = xhr.responseXML;
        emitWantedList(extDoc); 
      } else {                         // Error
        throw FireGestures._getLocaleString("XMLHTTPREQ_ERROR");
      }
    }
  };
  xhr.onerror = function (e) {
    throw FireGestures._getLocaleString("XMLHTTPREQ_ERROR");
  };
  xhr.send(null);
}


// Main Script (Entry Point) //////////////////////////////////////////////////////////////////////

// Check if the page is a bricklink wanted list
if (!PAGE_RESP.test(PAGE_URL)) {
  throw FireGestures._getLocaleString("NO_BRICKLINK_WANTED_LIST");
  return;
}

// Generate a clean URL plus a wantedsize tail (allows all parts in a single page)
var plusURL = PAGE_URL.replace(CLEAR_URL_RESP, "") + "&wantedSize=1000";

// Reload the page appending &WantedSize=1000 to the URL 
// Requires asynchronous call to the page. 
loadHTMLPage(plusURL); 
