// ==UserScript==
// @name           BrickLink Wanted List Exporter
// @namespace      http://www.smallapple.net/
// @description    Export BrickLink Wanted List
// @author         Ng Hun Yang, modified by Me
// @include        http://*.bricklink.com/wantedDetail.asp*
// @include        http://*.bricklink.com//wantedDetail.asp*
// @include        https://*.bricklink.com/wantedDetail.asp*
// @include        https://*.bricklink.com//wantedDetail.asp*
// @grant          none
// @match          *://*.bricklink.com/wantedDetail.asp*
// @version        1.13.0
// ==/UserScript==

/* Tested on Firefox 47.0, Chrome 26 and Opera 12.15 */

(function() {

var win = typeof(unsafeWindow) != "undefined" ? unsafeWindow : window;
var doc = win.document;

var dom = {};
var hlp = {};

dom.gE = function(id) {
  return doc.getElementById(id);
  };

dom.cE = function(tag) {
  return doc.createElement(tag);
  };

dom.cT = function(s) {
  return doc.createTextNode(s);
  };

dom.attr = function(obj, k, v) {
  if(arguments.length == 2)
    return obj.getAttribute(k);

  obj.setAttribute(k, v);
  };

dom.append = function(obj, child) {
  obj.appendChild(child);
  };

dom.html = function(obj, s) {
  if(arguments.length == 1)
    return obj.innerHTML;

  obj.innerHTML = s;
  };

dom.emitHtml = function(tag, attrs, body) {
  if(arguments.length == 2) {
    if(typeof(attrs) == "string") {
      body = attrs;
      attrs = {};
      }
    }

  var list = [];

  for(var k in attrs) {
    list.push(k + "='" + String(attrs[k]).replace(/'/g, "\\'") + "'");
    }

  var s = "<" + tag + " " + list.join(" ") + ">";

  if(body != null)
    s += body + "</" + tag + ">";
   
  return s;
  };

dom.addEvent = function(e, type, fn) {
  function mouseEvent(event) {
    if(this != event.relatedTarget && !dom.isAChildOf(this, event.relatedTarget))
      fn.call(this, event);
    }

  // Entry point
  if(e.addEventListener) {
    var effFn = fn;

    if(type == "mouseenter") {
      type = "mouseover";
      effFn = mouseEvent;
      }
    else if(type == "mouseleave") {
      type = "mouseout";
      effFn = mouseEvent;
      }

    e.addEventListener(type, effFn, /*capturePhase*/ false);
    }
  else
    e.attachEvent("on" + type, function() { fn(win.event); });
  };

// -----------------------------------------------------------------------------

function escTags(str) {
  return str.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
  }

// -----------------------------------------------------------------------------

var ID_PREFIX = "ujs-bl-export-wanted-list-";

var DIV_ID = ID_PREFIX + "div";
var SEL_ID = ID_PREFIX + "sel";
var BTN_ID = ID_PREFIX + "btn";

// -----------------------------------------------------------------------------

var STORE_ID = "ujs-bl-export-wanted-list-";

var user_params = {};

function loadOptions() {
  var wanted_list = localStorage[STORE_ID + "wanted_list"] || 0;
  
  var wanted_list_opts = doc.querySelector("#" + SEL_ID);
  wanted_list_opts.value = wanted_list;

  user_params.win_name = localStorage[STORE_ID + "win_name"] || "wanted_list" + (+new Date());
  }

function storeOptions(opts) {
  localStorage[STORE_ID + "wanted_list"] = opts.wanted_list;
  localStorage[STORE_ID + "win_name"] = opts.win_name;

  user_params.win_name = opts.win_name;
  }

// -----------------------------------------------------------------------------

(function() {

var DIV_STYLE = "background-color: #222; border: #fff 1px solid; border-radius: 3px; color: white; font-family: arial,helvetica,sans-serif; padding: 0.3em; position: fixed; top: 3.1em; right: 0.5em";

var SEL_STYLE = "margin: 0.3em";

var BTN_TEXT = "  Esporta la Lista  ";
var BTN_STYLE = "border-radius: 3px; cursor: pointer; margin: 0.3em";

var OPTS_STYLE = "border-top: #fff 1px solid; font-size: 80%; margin: 0.5em 0.25em 0.25em";

function createSel(divNode, list) {
  var selNode = dom.cE("select");
  selNode.id = SEL_ID;

  dom.attr(selNode, "style", SEL_STYLE);

  var s = "";

  for(var i = 0; i < list.length; ++i)
    s += dom.emitHtml("option", { value: list[i].value }, list[i].text);

  dom.html(selNode, s);

  dom.append(divNode, selNode);
  }

function createBtn(divNode) {
  var btnNode = dom.cE("input");
  btnNode.id = BTN_ID;

  dom.attr(btnNode, "type", "button");
  dom.attr(btnNode, "value", BTN_TEXT);
  dom.attr(btnNode, "style", BTN_STYLE);

  dom.addEvent(btnNode, "click", emitWantedList); 

  dom.append(divNode, btnNode);
  }

function createDiv() {
  var divNode = dom.cE("div");
  divNode.id = DIV_ID;

  dom.attr(divNode, "style", DIV_STYLE);

  dom.append(doc.body, divNode);

  createSel(divNode, getWantedLists());
  dom.append(divNode, dom.cE("br"));
  createBtn(divNode);

  loadOptions();
  }

function getWantedLists() {
  var wantedListNodes = doc.querySelectorAll("input[name=wantedMoreIDmove]");
  var wantedListNameNodes = wantedListNodes[0].parentNode.querySelectorAll("b");

  var wantedList = [];

  for(var i = 0; i < wantedListNodes.length; ++i)
    wantedList.push({ value: wantedListNodes[i].value, text: wantedListNameNodes[i].textContent });

  return wantedList;
  }

if(self === top && doc.querySelector("form[name=wantedForm]"))
  createDiv();

}) ();

// -----------------------------------------------------------------------------

function getParts() {
  function get_elm_id(row) {
    if(row.children.length < 2)
      return false;

    var hElm = row.children[1].querySelector("a");
    if(!hElm)
      return false;

    if(!hElm.getAttribute("href").match(/\?wantedID=(\d+)$/))
      return false;

    return RegExp.$1;
  }

  function get_part_info(row) {
    if(row.children.length < 3)
      return false;

    var hElm = row.children[2].querySelector("a");
    if(!hElm) 
      return false;

    var PARTS_RESP = /\?(\w+)=([0-9a-zA-Z-]+)[^=]+=([0-9]+)$/
    var OTHER_RESP = /\?(\w+)=([0-9a-zA-Z-]+)$/
    var tType = "";
    var tId = "";
    var tColour = "";

    if(hElm.getAttribute("href").match(PARTS_RESP)) {
      tType = RegExp.$1;
      tId = RegExp.$2;
      tColour = RegExp.$3;
    } else if(hElm.getAttribute("href").match(OTHER_RESP)) {
      tType = RegExp.$1;
      tId = RegExp.$2;
      tColour = "0";
    } else {
      return false;
    }
    return { type: tType, id: tId, colour: tColour};  
  }
  
  // Entry point 
  var form = doc.querySelector("form[name=wantedForm]");
  var tbl = form.parentNode;
  var rows = tbl.querySelectorAll("tr");
  
  var list = [];

  for(var i = 1; i < rows.length; ++i) {
    var row = rows[i];

    var part_info = get_part_info(row);
    if(!part_info)
      continue;

    row = rows[i + 1];

    var notify = row.children[0].querySelectorAll("input").item(1).checked ? "Y" : "N";

    var hElm = row.children[1].querySelector("select");
    var cond = hElm.value;

    var min_qty = row.children[2].querySelectorAll("input").item(1).value;
    var max_price = row.children[3].querySelectorAll("input").item(1).value;
    var remarks = row.children[4].querySelector("input").value;

    list.push({
      part_type: part_info.type,
      part_id: part_info.id,
      color: part_info.colour, 
      cond: cond,
      min_qty: min_qty,
      max_price: max_price,
      remarks: remarks,
      notify: notify
      });
    }

  return list;
  }

function genWantedList(list, wanted_list) {
  function emit(tag, v) {
    return "<" + tag + ">" + v + "</" + tag + ">";
    }

  // Entry point
  var out_list = [];

  for(var i = 0; i < list.length; ++i) {
    var elm = list[i];
    var str = "";
    str += emit("ITEMTYPE", elm.part_type);
    str += emit("ITEMID", elm.part_id);
    str += emit("COLOR", elm.color);
    str += emit("CONDITION", elm.cond);
    str += emit("MINQTY", escTags(elm.min_qty));

    if(elm.max_price != "")
      str += emit("MAXPRICE", escTags(elm.max_price));

    if(elm.remarks != "")
      str += emit("REMARKS", escTags(elm.remarks));

    if(elm.notify != "N")
      str += emit("NOTIFY", elm.notify);

    if(wanted_list != 0)
      str += emit("WANTEDLISTID", wanted_list);

    out_list.push(emit("ITEM", str));
    }

  return emit("INVENTORY", "\n" + out_list.join("\n") + "\n");
  }

function emitWantedList() {
  var win_name = user_params.win_name;
  var title = "My Wanted List";

  win_name = "wanted_list" + (+new Date());

  var selNode = dom.gE(SEL_ID);
  var wanted_list = selNode.value;

  storeOptions({
    wanted_list: wanted_list,
    win_name: win_name
    });

  var new_win = window.open("", win_name); 
  var new_doc = new_win.document;

  var str = genWantedList(getParts(), wanted_list); 
  var head = "<html><head><title>" + title + "</title></head><body><pre>" 
  var tail = "</pre></body></html>"

  new_doc.body.innerHTML = head + escTags(str) + tail;
  new_doc.close();
  }

}) ();